//============================================================================
// ParseEval.g - Parse and Evaluate
//
// Author:  Bary W Pollack
// Date:    Mar. 06, 2004 - 17:00
//
// This example shows how one may include semantics directly into a parser
// to achieve immediate interpretation of results
//
//============================================================================

// Import the necessary classes
{
import java.io.*;
import java.util.*;
}


//----------------------------------------------------------------------------
// Define a Parser, call it ParseEvalParser
//----------------------------------------------------------------------------

class ParseEvalParser extends Parser;
{
    public static void main(String[] args) throws Exception {
    ParseEvalLexer  lexer  = new ParseEvalLexer(System.in);
    ParseEvalParser parser = new ParseEvalParser(lexer);
    int nValue = parser.expr();
    System.out.println();
    System.out.println("* Success; nValue = " + nValue + " *");
    System.out.println();
    }
}


//==================================================================
// ParseEval Syntax - simplified arithmetic expressions
//==================================================================

expr returns [ int nValue = 0 ]
{ int x; }
    :  nValue=mexpr
       ( PLUS  x=mexpr  { nValue += x; }
       | MINUS x=mexpr  { nValue -= x; }
       )*
    ;      

mexpr returns [ int nValue = 0 ]
{ int x; }
    :  nValue=atom ( STAR x=atom { nValue *= x; } )*
    ;    

atom returns [ int nValue = 0 ]
    :  i:INT { nValue = Integer.parseInt(i.getText()); }
    |  LPAREN nValue=expr RPAREN
    ;


//----------------------------------------------------------------------------
// The ParseEval Scanner
//----------------------------------------------------------------------------

class ParseEvalLexer extends Lexer;

options {
    charVocabulary = '\0'..'\377';
    k = 2;
}

LPAREN : '(' ;
RPAREN : ')' ;
PLUS   : '+' ;
MINUS  : '-' ;
STAR   : '*' ;

INT    :  ('0'..'9')+ ;

WS     : ( ' '
         | '\r' '\n'
         | '\n'
         | '\t'
         )
         {$setType(Token.SKIP);}
       ;    


// EOF
